<?php
//a function dump for functions used only for setting up and, although crucial, are never used again in the script

function is_connect() {
	$return = false;
	
	if (class_exists('iSDK')) {
		$app = new iSDK();
		
		if ($app->cfgCon("conn")) {
			$return = $app;
		}
	}
	
	return $return;
}

function check_upgrade_directory_permissions() {
	$extract_dir = dirname($_SERVER['DOCUMENT_ROOT'] . $_SERVER['SCRIPT_NAME']);
	
	if (!is_writeable($extract_dir)) {
		add_fatal_error('Please make sure that the entire "is_sync_importer" directory is writeable by the server as the config files need to be written and updated regularly in order for this script to function.');
	}
}

function set_internal_config() {
	global $internal_config;
	
	if (!is_writeable(INCLUDES)) {
		add_fatal_error('Please make sure that the "includes" directory is writeable by the server if not you will not be able to automatically upgrade to new versions.');
	} else {
		if (!file_exists(INTERNAL_CONFIG_PATH)) {
			$config = array(
				'script_first_run'=>time()
				, 'script_last_run'=>time()
				, 'call_home_last_run'=>time()
				, 'num_executions'=>1
				, 'start_from_unattached_customer_id'=>0
				, 'start_from_order_id'=>0
				, 'total_contacts_imported'=>0
				, 'total_orders_imported'=>0
				, 'activated'=>0
				, 'activated_date'=>''
				, 'activation_email'=>''
				, 'activation_key'=>''
				, 'in_progress'=>0
				, 'initial_customer_sync_complete'=>0
				, 'initial_order_sync_complete'=>0
			);
			
			if (!file_put_contents(INTERNAL_CONFIG_PATH, json_encode($config))) {
				add_fatal_error('This script uses an internal config file to function. Unable to write a file to ' . INCLUDES . 'internal_config.json');
			} else {
				chmod(INTERNAL_CONFIG_PATH, 0777);
			}
		}
		
		$internal_config = json_decode(file_get_contents(INTERNAL_CONFIG_PATH));
	}
	
}

function check_in_progress() {
	global $internal_config;
	
	if ($internal_config->in_progress && !isset($_GET['force_unlock'])) {
		add_fatal_error('You can not run this right now as another version of the script is in progress. Please try again in a minute or two');
	}
}

function curl_remote_get($req_url) {
    $client = curl_init($req_url);
    curl_setopt($client, CURLOPT_RETURNTRANSFER, 1);
    $return = curl_exec($client);
    
    curl_close($client);
    
    return $return;
}

function get_activation_form() {
	$return = false;
	
	handle_deactivation();
	handle_activation();
	
	if (!is_activated()) {
		$return = '<form method="POST" class="activation_form">
				<p><label><span>Email Address (used to purchase):</span><input type="text" name="activation_email" value="" /></label></p>
				<p><label><span>License Key (found in your email receipt):</span><input type="text" name="activation_key" value="" /></label></p>
				<p><input type="submit" name="activate_script" value="Activate" /></p>
			</form>';
	}
	
	return $return;
}

function is_activated() {
	global $internal_config;
	
	return (int)@$internal_config->activated;
}

function handle_deactivation() {
	global $internal_config, $product_id, $config_vars;
	
	if (isset($_REQUEST['deactivate_script'])) {
		if ($internal_config->activated != 0) {
			if ($url = $config_vars['auth_server_url']) {
				$request = rtrim($url, '/') . '/?wc-api=software-api&request=deactivation&instance=' . urlencode($internal_config->activation_instance) . '&email=' . urlencode($internal_config->activation_email) . '&product_id=' . urlencode($product_id) . '&licence_key=' . urlencode($internal_config->activation_key);
			
				if ($data = curl_remote_get($request)) {
					$data = json_decode($data);
					
					if ($data->reset == 1) {
						$internal_config->activated = 0;
						$internal_config->activated_date = '';
						$internal_config->activation_email = '';
						$internal_config->activation_key = '';
						
						//printr($data);
						add_fatal_error('Deactivate complete. You will now be asked to enter your license information before you will be able to use this again');
					} else {
						add_fatal_error('Deactivate failed.. Bad request to server');
					}
				}
			}
		} else {
			add_fatal_error('Deactivate failed.. The script has already previously been deactivated.');
		}
	}
}

function handle_activation() {
	global $internal_config, $product_id, $config_vars;
	//get local setting.. is_activated
	
	$return = false;
	
	if (isset($_POST['activate_script']) && !$internal_config->activated) {
		//if no.. activate
		$key = @$_POST['activation_key']; //'is_sync_0a6f035f-282b-4808-bc21-35eeeb1a9ff2';
		$email = @$_POST['activation_email']; //'barton.sean@gmail.com';
		
		if (!$email || !$key) {
			add_fatal_error('Please complete all fields to continue.');
		} else {
			
			if (!$url = $config_vars['auth_server_url']) {
				add_fatal_error('You need to have a line for auth_server_url in your config file and it not be empty. The script can not function until this is present');
			} else {
				$request = rtrim($url, '/') . '/?wc-api=software-api&request=activation&instance=' . urlencode($_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF']) . '&email=' . urlencode($email) . '&product_id=' . urlencode($product_id) . '&licence_key=' . urlencode($key);

				if ($data = curl_remote_get($request)) {
					$data = json_decode($data);
					
					//printr($data);
					
					if (is_object($data)) {
						if (!$data->activated) {
							add_fatal_error('Activation was not successful. Server returned the following message: <strong>' . $data->error . '</strong>');
						} else {
							$internal_config->activated = $data->activated;
							$internal_config->activated_date = date('Y-m-d H:i:s');
							$internal_config->activation_email = $email;
							$internal_config->activation_key = $key;
							$internal_config->activation_instance = $data->instance;
							
							save_local_config();
							
							call_home(true);
						}
					} else {
						add_fatal_error('Server returned an unexpected result. Please contact the script vendor for assistance. Output: ' . print_r($data, true));
					}
				} else {
					add_fatal_error('Server returned an unexpected result. Please contact the script vendor for assistance. Output: ' . print_r($data, true));
				}
			}
		}
	}
	
	return $return;
}


function upgrade_core($file_url) {
	global $product_id;
	
        $client = curl_init($file_url);
        curl_setopt($client, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($client, CURLOPT_TIMEOUT, 3600);
        $file = curl_exec($client);
	curl_close($client);
        
	$script_dir = dirname($_SERVER['DOCUMENT_ROOT'] . $_SERVER['SCRIPT_NAME']) . '/';
	$extract_dir = $script_dir . 'upgrades/' ;
	$backups_dir = $script_dir . 'upgrade_backups/' ;
	
	@mkdir($backups_dir, 0777, 1);
	chmod_directory($backups_dir);
	
	remove_directory($extract_dir);
	echo '<li>Removing Directory: ' . $extract_dir . '</li>';
	$i = (int)@mkdir($extract_dir, 0777, 1);
	chmod_directory($extract_dir);
	echo '<li>Creating Upgrades Directory ' . $i . '</li>';
	
	echo '<li>Uploading ZIP to ' . $extract_dir . '</li>';
        if (file_put_contents($extract_dir . $product_id . '.zip', $file)) {
		chmod_directory($extract_dir . $product_id . '.zip');
		
                $zip = new ZipArchive();  
                if ($zip->open($extract_dir . $product_id . '.zip')) {
			
			if ($i = (int)$zip->extractTo($extract_dir)) {
				echo '<li>Extracting ZIP to ' . $extract_dir . '. ' . $i . '</li>';
				
				chmod_directory($extract_dir);
				$zip->close();
				
				if ($i = unlink($extract_dir . $product_id . '.zip')) {
					echo '<li>Deleting upgrade ZIP file ' . (int)$i . '</li>';
					
					$old_includes_backup_dir = $backups_dir . 'includes_' . date('Ymd') . '_' . date('his');
					if ($i = (int)rename($script_dir . 'includes', $old_includes_backup_dir)) {
						echo '<li>Renaming old "includes" directory: ' . $old_includes_backup_dir . '. ' . $i . '</li>';
						chmod_directory($old_includes_backup_dir);
						
						if ($i = (int)rename($extract_dir . $product_id . '/includes', $script_dir . 'includes')) {
							chmod_directory($script_dir);
							
							echo '<li>Moving includes directory from upgrades to ZIP to ' . $script_dir . 'includes ' . (int)$i . '</li>';
							
							remove_directory($extract_dir);
							die('<li>Upgrade Complete!</li>');
						} else {
							echo '<li>Unable to move includes directory from "' . $extract_dir . $product_id . '/includes" to ZIP to ' . $script_dir . 'includes ' . (int)$i . '</li>';
							
							if ($i = (int)rename($script_dir . 'includes', $old_includes_backup_dir)) {
								echo '<li>Restoring includes directory from backup: "' . $old_includes_backup_dir . 'includes" to ' . $script_dir . 'includes ' . (int)$i . '</li>';
							} else {
								echo '<li>Unable to restore includes directory from backup. Manually restore "' . $old_includes_backup_dir . '" for the script to function again</li>';
							}
						}
					} else {
						echo '<li>Unable to archive existing includes directory from ' . $script_dir . 'includes into ' . $old_includes_backup_dir . '</li>';
					}
				} else {
					echo '<li>Unable to remove upgrade ZIP: ' . $extract_dir . $product_id . '.zip</li>';
				}
			} else {
				echo '<li>Unable to extract upgrade ZIP: ' . $extract_dir . $product_id . '.zip. ' . $zip->getStatusString() . '</li>';
			}
		} else {
			echo '<li>Unable to open ZIP file</li>';
		}
        } else {
		echo '<li>Unable to place file contents into temp file</li>';
	}
	
	//remove_directory($extract_dir);
	die('<li>Upgrade Failed!</li>');
}

function is_locked() {
	global $internal_config;
	
	$return = false;
	
	if ($internal_config->in_progress) {
		$return = true;
	}
	
	return $return;
}

function call_home($force=false) {
	global $config_vars;
	
	$return = false;
	
	if ($url = $config_vars['auth_server_url']) {
		//CURL home here according to the API docs.
		global $internal_config, $product_id, $config_vars;

		if ((int)@$internal_config->call_home_last_run > (time()-86400) && !isset($_REQUEST['force_call_home']) && !$force && !isset($_REQUEST['issi_client_action'])) {
			$return = true;
		} else {
			$request = rtrim($url, '/') . '/?wc-api=software-api&request=call_home&instance=' . urlencode($internal_config->activation_instance) . '&version_number=' . urlencode(VERSION_NUMBER) . '&licence_key=' . urlencode($internal_config->activation_key);
			
			if ($return = curl_remote_get($request)) {
				$internal_config->call_home_last_run = time();
				
				if (isset($_REQUEST['issi_client_action'])) {
					if ($_REQUEST['issi_client_action'] == 'remote_upgrade') {
						if (!is_locked()) {
							if ($_REQUEST['issi_api_key'] == $internal_config->activation_key) {
								if ($upgrade_file = $_REQUEST['issi_file']) {
									upgrade_core($upgrade_file);
								} else {
									add_fatal_error('No upgrade file passed');
								}
							} else {
								add_fatal_error('Upgrade failed as API keys do not match');
							}
						} else {
							add_fatal_error('You can not currently upgrade the site because it is being run by another script. Please try again shortly.');
						}
					} else {
						add_fatal_error('Unknown client action.');
					}
				}
				
				if (isset($_REQUEST['force_call_home'])) {
					die('Call Home Complete');
				}
			}
		}
	}
	
	return $return;
}

function check_remote_request() {
	global $config_vars, $internal_config;
	
	
}

function check_version() {
	global $config_vars;
	
	$return = false;
	
	if ($config_vars['version_server_url']) {
		//VERSION_NUMBER - send this home to compare versions. later versions might want to force an upgrade
		//CURL home here according to the API docs. returns the version number of the appropriate version
		//$return = 2.1; //for testing purposes leave in
	}
	
	return $return;
}

function output_version_check() {
	return ''; //disabled for now

	$return = 'You have the latest version';
	$new_version_number = check_version();
	
	if ($new_version_number && $new_version_number != VERSION_NUMBER) {
		$return = 'new version available. ' . $new_version_number;
	}
	
	return '(' . $return . ')';
}

function check_module() {
	global $config_vars;
	
	$return = false;
	
	if ($config_vars['module']) {
		$return = file_exists(MODULES . $config_vars['module'] . '.class.php');
	}
	
	return $return;
}

function check_zip_module() {
	return class_exists('ZipArchive');
}

function check_manual_throttle() {
	global $config_vars;
	
	$return = true;
	
	if ($config_vars['batch_limit'] < 5 || $config_vars['batch_limit'] > 100) {
		$return = false;
	}
	
	return $return;
}

function check_config() {
	global $config_vars;
	
	if (count($config_vars)) {
		$return = true;
	} else {
		$return = false;
	}
	
	
	$config_excludes = array('update_old_addresses', 'admin_email', 'additional_name_formats', 'secondary_search_format', 'order_status_cf_id');
	
	foreach ($config_vars as $key=>$var) {
		if (is_array($var)) {
			if (!check_config($var)) {
				$return = false;
			}
		} else if ($var == '' && !in_array($key, $config_excludes)) {
			$return = false;
		}
	}
	
	return $return;
}

function check_log_table() {
	
	$sql = 'CREATE TABLE IF NOT EXISTS is_import_log (
			id INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY
			, customers_imported INT(11)
			, orders_imported INT(11)
			, created_date INT(11) NOT NULL
		);';
	db_query($sql);
}

function get_modules() {
	if ($modules = scandir(MODULES)) {
		$new_modules = array();
		
		foreach ($modules as $i=>$module) {
			if (!in_array($module, array('.', '..', '.htaccess', 'module.class.php'))) {
				$new_modules[] = str_replace('.class.php', '', $module);
			}
 		}
		
		$modules = $new_modules;
	}
	
	return $modules;
}

function get_module_select($name, $selected=false) {
	$html = '';
	
	if ($modules = get_modules()) {
		$html .= '<select name="' . $name . '" class="select_module_' . $module . '">';
		
		foreach ($modules as $i=>$module) {
			$html .= '<option value="' . $module . '" ' . ($selected == $module ? 'selected="selected"':'') . '>' . ucwords(str_replace('_', ' ', $module)) . '</option>';
		}
		
		$html .= '</select>';
	}
	
	return $html;
}

function setup_wizard() {
	$return = '';
	
	$return .= '<p class="wizard_setup">As this is your first run we need to generate the config file. Please complete the following fields to start using the system. If you have any queries as to the operation of this script please contact the vendor.</p>';
	
	$return .= '<form method="POST" class="wizard_form">
		<p><label><span>Infusionsoft Application Name:</span><input type="text" name="config[is_app_name]" value="" /></label></p>
		<p><label><span>Infusionsoft API Key:</span><input type="text" name="config[is_api_key]" value="" /></label></p>
		<p><label><span>What eCommerce platform are you using?:</span>' . get_module_select('config[module]') . '</label></p>
		<p><label><span>How many orders should be imported each time this script is run (25 recommended):</span><input type="text" name="config[batch_limit]" value="" /></label></p>
		<p><label><span>A custom field should be created in IS for Order statuses. What is the DB name of this field? Leave blank or call ISL for help (optional):</span><input type="text" name="config[order_status_cf_id]" value="" /></label></p>
		<p><label><span>Old importer URL format (<small>If the old orders read "Order imported by Universal Importer from http://www.balloontwisting.co.uk/shop - Id #: 100000001" then add the URL only as follows including any http and any slashes used "http://www.balloontwisting.co.uk/shop"</small>):</span><input type="text" name="config[secondary_search_format]" value="" /></label></p>
		<p><label><span>Should addresses for orders more than a day old be updated?</span><select name="config[update_old_addresses]"><option value="0">No</option><option value="1">Yes</option></select></label></p>
		<p><input type="submit" name="save_config" value="Save Configuration" /></p>
	</form>';
	
	$return .= '<p class="cron_setup">As part of the recurring aspect of this script you will need to set up a CRON job (scheduled task). Please run this script every 5-15 minutes to autmatically sync your site.</p>';
	
	return user_feedback($return, true);
}

?>